#include "stdafx.h"
#include "dispatch.h"
#include "ioctls.h"
#include "list.h"

// just allow the open
NTSTATUS DHeapDmpDispatchCreate(PDEVICE_OBJECT /*pDevObj*/, PIRP pIrp)
{
	PAGED_CODE();
	pIrp->IoStatus.Status = STATUS_SUCCESS;
	pIrp->IoStatus.Information = 0;
	IoCompleteRequest(pIrp, IO_NO_INCREMENT);
	return STATUS_SUCCESS;
}

// and likewise the close
NTSTATUS DHeapDmpDispatchClose(PDEVICE_OBJECT /*pDevObj*/, PIRP pIrp)
{
	PAGED_CODE();
	pIrp->IoStatus.Status = STATUS_SUCCESS;
	pIrp->IoStatus.Information = 0;
	IoCompleteRequest(pIrp, IO_NO_INCREMENT);
	return STATUS_SUCCESS;
}

// disable the warning
// C4509: nonstandard extension used: 'DHeapDmpDispatchControl' uses SEH and 'lock' has destructor
// the generated code does the right thing
#pragma warning(push)
#pragma warning(disable : 4509)
NTSTATUS DHeapDmpDispatchControl(PDEVICE_OBJECT /*pDevObj*/, PIRP pIrp)
{
	PAGED_CODE();
	NTSTATUS stat = STATUS_SUCCESS;
	PIO_STACK_LOCATION pIoStack = IoGetCurrentIrpStackLocation(pIrp);
	PIO_STATUS_BLOCK pIoStatus = &pIrp->IoStatus;
	// initialize to sensible defaults
	pIoStatus->Information = 0;
	{
		// they'll be a prefast warning here
		// as the version in the win7 wdk 
		// doesn't seem to think that destructors actually run.
		// The constructor and destructor are properly annotated
		// but it makes no difference
		FastMutexLock lock(&g_Mutex);
		__try
		{
			switch(pIoStack->Parameters.DeviceIoControl.IoControlCode)
			{
				case IOCTL_DHEAPMON_COUNT_DESKTOP:
				{
					stat = GetDesktopCount(pIrp, &pIoStatus->Information);
				}
				break;
				case IOCTL_DHEAPMON_GET_DESKTOP_INFO:
				{
					stat = GetDesktopInfo(pIrp, &pIoStatus->Information);
				}
				break;
				default:
				{
					DBG_TEXT("Receieved invalid IOCTL 0x%x", pIoStack->Parameters.DeviceIoControl.IoControlCode);
					stat = STATUS_INVALID_PARAMETER;
				}
				break;
			}
		}
		__except(stat = GetExceptionCode(), EXCEPTION_EXECUTE_HANDLER)
		{
			EmptyDeskList();
			pIoStatus->Information = 0;
		}
	}
	pIoStatus->Status = stat;
	IoCompleteRequest(pIrp, IO_NO_INCREMENT);
	return stat;
}
#pragma warning(pop)
